function [A,F,B1,B2,Pv,C, R ,r,V] = NKexample

% Benhabib-Farmer  Supply curve with Indeterminacy
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file produces input for the program Findequiv.  It generates an
% example for the working paper "What we don't know about the
% monetary transission mechanism and why we don't know it"
% (November 2004 (original version February 2004)) 
% by Andreas Beyer and Roger E. A. Farmer.
%
% The DGP is a standard New Keynesian model.  The IS curve 
% contains habit formation. The policy rule contains future inflation and 
% the lagged interest rate. The AS curve contains only future inflation and 
% current unemployment.  
%
% The econometrician fits a Benhabib Farmer model. The IS curve is the correct one. 
% The AS curve contains lagged unemployment and the lagged and current interest rate. 
% The estimated policy rule contains current inflation and the lagged interest rate. 
%
% The main point of this example is that the DGP is determinate but the estimated 
% model is indeterminate.  The matrice A F B1 B2 C and Pv are the
% determinate NK data generation process. V is the vcv matrix of the shocks. 
% The Default is the identity matrix  The matrice R and r, produced by
% the program satisfy the condition R*vec([Abar Fbar -B1bar -B2bar -Cbar -Pvbar]')=r. 
% They identify a Benhabib-Farmer model.  The program FindEquiv takes the
% output of this program and finds the matrices Abar Fbar Babar B2bar Cbar
% and Pvbar and the vcv matrix of the shocks such that the Benhabib Farmer
% model has the same reduced form as the NK DGP.
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Andreas Beyer and Roger E. A. Farmer
% November 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

a11=1;a12=0;a13=0.05;   %this the interest rate coeffcient in y eqn
a21=-0.5;a22=1;a23=0;  %this is the y coefficient in the PC
a31=0;a32=0;a33=1;      %these are current policy rule coefficients

f11=-0.5;f12=-a13;f13=0;
f21=0;f22=-0.8;f23=0;
f31=0;f32=-1.1;f33=0;

b11=0.5;b12=0;b13=0;
b21=0;b22=0.25;b23=0;
b31=0;b32=0;b33=0.8;

c1=0.0; %assume 0 output gap
c2=.02; %assume 2% target inflation rate
realrate=.03; %assume 3% natural real interest rate
c3=realrate+c2; % this is the steady state i rate

A=[a11 a12 a13 ;a21 a22 a23 ;a31 a32 a33];
F=[f11 f12 f13;f21 f22 f23;f31 f32 f33];
B1=[b11 b12 b13;b21 b22 b23;b31 b32 b33];
B2=0*B1;
C=(A+F-B1)*[c1;c2;c3];
V=eye(size(A,1));
Pv=eye(3);

n=3;
% This section sets the restrction matrix
% Identification requires 18 restrictions

% These rows are for the output gap equation
%    y  p  i  y'  p'  i'  yL  pL  iL  EyL EpL EiL  c  v1  v2  v3       
R11=[1  0  0   0  0   0   0   0   0    0   0   0   0   0   0   0  ];r11=1;
R12=[0  1  0   0  0   0   0   0   0    0   0   0   0   0   0   0  ];r12=0;
R13=[0  0  1   0  1   0   0   0   0    0   0   0   0   0   0   0  ];r13=0;
R14=[1  0  0   1  0   0  -1   0   0    0   0   0   0   0   0   0  ];r14=0;
R15=[0  0  0   0  0   0   0   0   1    0   0   0   0   0   0   0  ];r15=0;
R16=[0  0  0   0  0   1   0   0   0    0   0   0   0   0   0   0  ];r16=0;


R1=[R11;R12;R13;R14;R15;R16];
r1=[r11;r12;r13;r14;r15;r16];
row1=size(R1);
R1=[R1 zeros(row1) zeros(row1)];

% These rows are for the inflation equation
%    y  p  i  y'  p'  i'  yL  pL  il  EyL EpL EiL  c  v1  v2  v3           
R21=[1  0  0   0  0   0   0   0   0    0   0   0   0   0   0   0 ];r21=1;
R22=[0  1  0   0  0   0   0   0   0    0   0   0   0   0   0   0 ];r22=0;
R23=[0  0  0   1  0   0   0   0   0    0   0   0   0   0   0   0 ];r23=0;
R24=[0  0  0   0  0   1   0   0   0    0   0   0   0   0   0   0 ];r24=0;
R25=[0  0  0   0  1   0   0   0   0    0   0   0   0   0   0   0 ];r25=-0;
R26=[0  0  0   0  0   0   0   1   0    0   0   0   0   0   0   0 ];r26=-0;


R2=[R21;R22;R23;R24;R25;R26];
r2=[r21;r22;r23;r24;r25;r26];
row2=size(R2);
R2=[zeros(row2) R2 zeros(row2) ];

% These rows are for the policy equation
%    y  p  i  y'  p'  i'  yL  pL  il  EyL EpL EiL  c  v1  v2  v3          
R31=[0  0  1   0  0   0   0   0   0    0   0   0   0   0   0   0    ];r31=1;
R32=[0  0  0   0  0   0   1   0   0    0   0   0   0   0   0   0    ];r32=0;
R33=[0  0  0   1  0   0   0   0   0    0   0   0   0   0   0   0    ];r33=0;
R34=[0  0  0   0  0   1   0   0   0    0   0   0   0   0   0   0    ];r34=0;
R35=[0  0  0   0  1   0   0   0   0    0   0   0   0   0   0   0    ];r35=0;
R36=[0  0  0   0  0   0   0   1   0    0   0   0   0   0   0   0    ];r36=0;

R3=[R31;R32;R33;R34;R35;R36];
r3=[r31;r32;r33;r34;r35;r36];
row3=size(R3);
R3=[ zeros(row3) zeros(row3) R3 ];
R=[R1;R2;R3];r=[r1;r2;r3];
%%%%%%%%%%%%%%%%%%%%%%%%%  END %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%