function [W] = VCVsolve(A,B,V)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function takes as inputs a k1*k1 VCV matrix V, an n*k1 matrix A and an n*k2
% matrix of coefficients B where k2 >= k1.  It delivers a k2*k2 matrix W
% that solves the equation A*V*A' = B*W*B'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Andreas Beyer and Roger E. A. Farmer
% February 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if size(A,2)>size(B,2);W=[];disp('Error: No Solution: Not enough shocks');return;end;
n=size(A,1);     
if nargin == 2; V=eye(n); end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% m is the number of shocks to the transformed model; this may include
% sunspots. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
m=size(B,2);
A1t=kron(A,A);
B1t=kron(B,B);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE INDICES OF DIAGONAL AND OFF DIAGONAL ELEMENTS of Vec(V)
% ind is a set of indices of elements of vec(V) that correspond to upper
% triangular elements of the V matrix.  The remaining elements are redundant
% by symmetry.  ind is also used to index the relevant rows of Equation (1).
% indDiag are the indices of the elements of vec(V) that are on the diagonal 
% notinda are the indices of offdiagonal elements below the diagonal
% notindb are indices of offdiagonal elememnts above the diagonal
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ind=1;
for j=1:n-1;
    row=j*(n)+1:j*(n)+j+1;
    row=row';
    ind=[ind;row];
end
id=(1:n);indDiag=(id-1)*n+id;
t1=1:n*n;
mat1=reshape(reshape(1:n*n,n,n)',n*n,1);
notinda=find(~ismember(t1,ind)); 
notindb=mat1(notinda)';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE INDICES OF DIAGONAL AND OFF DIAGONAL ELEMENTS of Vec(W)
% ind2 is a set of  indices of elements of vec(W) that correspond to upper
% triangular elements of the W matrix.  The remaining elements are redundant
% by symmetry.  
% indDiag2 are the indices of the elements of vec(W) that are on the diagonal 
% notinda2 are the indices of offdiagonal elements below the diagonal
% notindb2 are indices of offdiagonal elememnts above the diagonal
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ind2=1;
for j=1:m-1;
row=j*(m)+1:j*(m)+j+1;
row=row';
ind2=[ind2;row];
end
id2=(1:m);indDiag2=(id2-1)*m+id2;
t2=1:m*m;
mat2=reshape((reshape(1:m*m,m,m)'),m*m,1);
id2=(1:m);indDiag2=(id2-1)*m+id2;
t2=1:m*m;
mat2=reshape(reshape(1:m*m,m,m)',m*m,1);
notinda2=find(~ismember(t2,ind2)); %
notindb2=mat2(notinda2)';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code removes rows of A1 and B1 that correspond to the symmetric and
% therefore redundant elements of W.  It also removes redundant columns and 
% adds together columns that correspond to symmetric elements of W.  
% Thus: a12*sigma12 +  a21*sigma21 is replaced by (a12 + a21)*sigma12
% A1 and B1 satisfy the following equation:
% A1*vec(V)(ind) = B1*vec(W)(ind2)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
A2t=A1t;B2t=B1t;A3t=A2t;B3t=B2t;
A3t(ind,notindb)=A2t(ind,notindb)+A2t(ind,notinda);
A1=A3t(ind,ind);
B3t(ind,notindb2)=B2t(ind,notindb2)+B2t(ind,notinda2);
B1=B3t(ind,ind2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following code checks for linearly dependendent columns of the matrix B.
% Each linearly dependent column corresponds to an unidentified
% parameter of the vcv matrix.  For each linearly dependent column the
% program adds a row to B that sets the appropriate element of W to zero.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
B1b=[];bindex=[];r=[];B11=B1;
for i=1:size(B1,2);
    if rank([B1b B1(:,i)])>rank(B1b);
        B1b=[B1b B1(:,i)];
        bindex=[bindex;i];
    else
        B11=[B11;zeros(1,i-1) 1 zeros(1,size(B1,2)-i)];r=[r;0];
    end
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now we find the parameters of W that replicate the reduced form of the
% original model.  These are stored in the vector L.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
omegav=reshape(V,size(V,1)*size(V,2),1);omegav=omegav(ind);
r=[A1*omegav ; r];
L=inv(B11)*r;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Finally we pack the elements of L back into W
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
W=zeros(m*m,1);
W(ind2)=L;
W(notinda2,:)=W(notindb2,:);
W=reshape(W,m,m);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  END %%%%%%%%%%%%%%%%%%%%%%%%%%%%%